// UrsOTA
//
// Autor: http://UllisRoboterSeite.de
// Doku:  http://bienonline.magix.net/public/esp8266-ota.html

#include <UrsOTA.h>

void UrsOTAClass::begin(const char* HostName, int Port, const char* PassWord, Print * OutDevice) {
  ArduinoOTA.setHostname(HostName);
  ArduinoOTA.setPort(Port);
  ArduinoOTA.setPassword(PassWord);
  _OutDevice = OutDevice;

    ArduinoOTA.onStart([this]() {
     if (_OutDevice) _OutDevice->println(F("\nOTA Start"));
     if (_start_callback) _start_callback();
   });

    ArduinoOTA.onEnd([this]() {
      if (_end_callback) _end_callback();
      if (_OutDevice) _OutDevice->println(F("OTA beendet"));
    });

    ArduinoOTA.onProgress([this](unsigned int progress, unsigned int total)
    { uint8_t percentage = (progress / (total / 100));
    if (percentage > 0 && (percentage % 10) == 0)
      if (percentage != lastPercentage) {
        if (_progress_callback) _progress_callback(percentage);
        if (_OutDevice) _OutDevice->printf_P(PSTR("Fortschritt: %u%%\n"), percentage);
        lastPercentage = percentage;
      }
    });

    ArduinoOTA.onError([this](ota_error_t error) {
      if (_error_callback) _error_callback(error);
      if (_OutDevice) {
        _OutDevice->printf_P(PSTR("OTA Error [%u]: "), error);
        if (error == OTA_AUTH_ERROR) _OutDevice->println(F("Auth fehlgeschlagen"));
        else if (error == OTA_BEGIN_ERROR) _OutDevice->println(F("Begin fehlgeschlagen"));
        else if (error == OTA_CONNECT_ERROR) _OutDevice->println(F("Connect fehlgeschlagen"));
        else if (error == OTA_RECEIVE_ERROR)_OutDevice->println(F("Receive fehlgeschlagen"));
        else if (error == OTA_END_ERROR) _OutDevice->println(F("End fehlgeschlagen"));
      }
    });

  ArduinoOTA.begin();

  if (_OutDevice) {
    _OutDevice->printf_P(PSTR("OTA mit Hostname '%s' initialisiert.\n"), HostName);
  }
}

void UrsOTAClass::begin(const String HostName, const int Port, const String PassWord, Print * OutDevide) {
  begin(HostName.c_str(), Port, PassWord.c_str(), OutDevide);
}

void UrsOTAClass::begin(const char * HostName, Print * OutDevide) {
  begin(HostName, 0, NULL, OutDevide);
}
void UrsOTAClass::begin(const String HostName, Print * OutDevide) {
  begin(HostName.c_str(), 0, NULL, OutDevide);
}

void UrsOTAClass::onStart(UrsOtaHandlerFunction fn) {
  _start_callback = fn;
}

void UrsOTAClass::onEnd(UrsOtaHandlerFunction fn) {
  _end_callback = fn;
}

void UrsOTAClass::onProgress(UrsOtaHandlerFunction_Progress fn) {
  _progress_callback = fn;
}

void UrsOTAClass::onError(UrsOtaHandlerFunction_Error fn) {
  _error_callback = fn;
}

UrsOTAClass UrsOTA;

